@@ -1,22 +1,12 @@
-/* This file is part of the KDE libraries
-   Copyright (C) 2000 Kurt Granroth <granroth@kde.org>
-   Copyright (C) 2006 Hamish Rodda <rodda@kde.org>
-   Copyright     2007 David Faure <faure@kde.org>
+/*
+    This file is part of the KDE libraries
+    SPDX-FileCopyrightText: 2000 Kurt Granroth <granroth@kde.org>
+    SPDX-FileCopyrightText: 2006 Hamish Rodda <rodda@kde.org>
+    SPDX-FileCopyrightText: 2007 David Faure <faure@kde.org>
 
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
-   License version 2 as published by the Free Software Foundation.
-
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-
-   You should have received a copy of the GNU Library General Public License
-   along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-only
 */
+
 #include "kedittoolbar.h"
 #include "kedittoolbar_p.h"
 #include "debug.h"
@@ -145,7 +135,7 @@ public:
      */
     QString toolBarText(const QDomElement &it) const;
 
-    bool         m_isModified;
+    bool         m_isModified = false;
     ToolBarList &barList()
     {
         return m_barList;
@@ -347,8 +337,7 @@ IconTextEditDialog::IconTextEditDialog(QWidget *parent)
     setWindowTitle(i18nc("@title:window", "Change Text"));
     setModal(true);
 
-    QVBoxLayout *layout = new QVBoxLayout;
-    setLayout(layout);
+    QVBoxLayout *layout = new QVBoxLayout(this);
 
     QGridLayout *grid = new QGridLayout;
     grid->setContentsMargins(0, 0, 0, 0);
@@ -558,9 +547,7 @@ using namespace KDEPrivate;
 class KEditToolBarPrivate
 {
 public:
-    KEditToolBarPrivate(KEditToolBar *q): q(q),
-        m_accept(false), m_global(false),
-        m_collection(nullptr), m_factory(nullptr), m_widget(nullptr) {}
+    KEditToolBarPrivate(KEditToolBar *q): q(q) {}
 
     void init();
 
@@ -571,17 +558,17 @@ public:
     void applyClicked();
     void defaultClicked();
 
-    KEditToolBar *q;
-    bool m_accept;
+    KEditToolBar *const q;
+    bool m_accept = false;
     // Save parameters for recreating widget after resetting toolbar
-    bool m_global;
-    KActionCollection *m_collection;
+    bool m_global = false;
+    KActionCollection *m_collection = nullptr;
     QString m_file;
     QString m_defaultToolBar;
-    KXMLGUIFactory *m_factory;
-    KEditToolBarWidget *m_widget;
-    QVBoxLayout *m_layout;
-    QDialogButtonBox *m_buttonBox;
+    KXMLGUIFactory *m_factory = nullptr;
+    KEditToolBarWidget *m_widget = nullptr;
+    QVBoxLayout *m_layout = nullptr;
+    QDialogButtonBox *m_buttonBox = nullptr;
 };
 
 Q_GLOBAL_STATIC(QString, s_defaultToolBarName)
@@ -616,9 +603,7 @@ void KEditToolBarPrivate::init()
     q->setWindowTitle(i18nc("@title:window", "Configure Toolbars"));
     q->setModal(false);
 
-    m_layout = new QVBoxLayout;
-    q->setLayout(m_layout);
-
+    m_layout = new QVBoxLayout(q);
     m_layout->addWidget(m_widget);
 
     m_buttonBox = new QDialogButtonBox(q);
